@@ -1,6 +1,6 @@
 utils::globalVariables(c("id", "name", "pk", "wNbr", "wGrp", "pNbr", "use_nr",
   "add_txt_pk", "de", "fr", "it", "en", "exhaustionDeadline", "soldoutDeadline",
-  "isSalePermission", "terminationReason",
+  "isSalePermission", "terminationReason", "substance_de",
   "packageInsert", "permission_holder", "producingCountryPrimaryKey",
   "ingredient_de", "ingredient_fr", "ingredient_it", "categories",
   "min_dosage", "max_dosage", "min_rate", "max_rate", "waiting_period",
@@ -25,7 +25,7 @@ srppp_xml_get <- function(from, ...)
 #' @examples
 #' # Try to get the current SRPPP as available from the FOAG website
 #' \donttest{
-#' srppp_cur <- try(srppp_xml_get())
+#' srppp_xml_cur <- try(srppp_xml_get())
 #' }
 srppp_xml_get.NULL <- function(from, ...)
 {
@@ -50,13 +50,33 @@ srppp_xml_get.character <- function(from, ...)
 #' @param path A path to a zipped SRPPP XML file
 #' @export
 srppp_xml_get_from_path <- function(path, from) {
-  zip_contents <- utils::unzip(path, list = TRUE)
-  xml_filename <- grep("PublicationData_20.._.._...xml",
-    zip_contents$Name, value = TRUE)
-  xml_con <- unz(path, xml_filename)
-  ret <- read_xml(xml_con)
-  class(ret) <- c("srppp_xml", "xml_document", "xml_node")
+  if (grepl("zip$", path)) {
+    zip_contents <- utils::unzip(path, list = TRUE)
+    xml_filename <- grep("PublicationData_20.._.._...xml",
+      zip_contents$Name, value = TRUE)
+    xml_con <- unz(path, xml_filename)
+    ret <- read_xml(xml_con)
+  } else {
+    if (grepl("xz$", path)) {
+      xml_con <- xzfile(path)
+      ret <- read_xml(xml_con)
+    }
+  }
+
+  # Determine the version of the XML file and attach it to the returned object
+  substance_type_node <- xml_find_first(ret,
+    "Products/Product/ProductInformation/Ingredient/SubstanceType")
+
+  if (is.na(xml_attr(substance_type_node, "SubstanceType"))) {
+    attr(ret, "srppp_xml_version") <- 1
+  } else {
+    attr(ret, "srppp_xml_version") <- 2
+  }
+
+  # Attach the "from" information as well
   attr(ret, "from") <- as.character(from)
+
+  class(ret) <- c("srppp_xml", "xml_document", "xml_node")
   return(ret)
 }
 
@@ -88,21 +108,27 @@ srppp_xml_get_products <- function(srppp_xml = srppp_xml_get(), verbose = TRUE,
 {
   product_nodeset <- xml_find_all(srppp_xml, "Products/Product")
   product_attribute_names <- names(xml_attrs(product_nodeset[[1]]))
+
+  product_pNbrs <- product_nodeset |>
+    xml_attrs() |>
+    unlist() |>
+    matrix(ncol = 7, byrow = TRUE,
+      dimnames = list(NULL, product_attribute_names)) |>
+    as_tibble() |>
+    filter(!grepl("-", wNbr)) |> # remove sales permissions
+    mutate(pNbr = as.integer(id)) |>
+    mutate(wGrp = wNbr) |> # we define a "W-Number group", i.e. the part of the W-Number before the dash
+    select(wGrp, pNbr)
+
   products_no_permission_holders <- product_nodeset |>
     xml_attrs() |>
     unlist() |>
     matrix(ncol = 7, byrow = TRUE,
       dimnames = list(NULL, product_attribute_names)) |>
     as_tibble() |>
-    mutate(wGrp = as.integer(gsub("-.*$", "", wNbr)),
-      .before = wNbr) |>
-    group_by(wGrp) |>
-    mutate(id = as.integer(id)) |>
-    mutate(pNbr = if_else(id < 38, NA, id),
-      .after = id) |>
-    tidyr::fill(pNbr) |>
-    ungroup() |>
+    mutate(wGrp = gsub("-.*$", "", wNbr)) |>
     mutate(isSalePermission = if_else(isSalePermission == "true", TRUE, FALSE)) |>
+    left_join(product_pNbrs, by = "wGrp") |>
     select(wNbr, name, pNbr, exhaustionDeadline, soldoutDeadline,
       isSalePermission, terminationReason)
 
@@ -116,7 +142,7 @@ srppp_xml_get_products <- function(srppp_xml = srppp_xml_get(), verbose = TRUE,
   }))
   colnames(ph_key_matrix) <- c("wNbr", "permission_holder")
   ph_keys <- as_tibble(ph_key_matrix)
-  ph_keys$permission_holder <- as.integer(ph_keys$permission_holder)
+  ph_keys$permission_holder <- ph_keys$permission_holder
 
   # Discard the second permission holder
   # The code was developed for ParallelImports, where this occurred
@@ -138,10 +164,16 @@ srppp_xml_get_products <- function(srppp_xml = srppp_xml_get(), verbose = TRUE,
       # Define the reference node (original product)
       ref_node <- xml_child(product_nodeset[[node_numbers[1]]])
       ref_contents <- unlist(strsplit(as.character(ref_node), "\n"))
-      ref_contents_permholder_i <- grep("PermissionHolderKey", ref_contents)
-      ref_contents_clean <- ref_contents[-ref_contents_permholder_i]
 
-      # Loop over sales permissions
+      # In certain cases (2025-12-01, W-Number 7255), there is no permission holder
+      if (any(grepl("PermissionHolderKey", ref_contents))) {
+        ref_contents_permholder_i <- grep("PermissionHolderKey", ref_contents)
+        ref_contents_clean <- ref_contents[-ref_contents_permholder_i]
+      } else {
+        ref_contents_clean <- ref_contents
+      }
+
+      # Loop over products
       for (i in 2:length(node_numbers)) {
         check_node <- xml_child(product_nodeset[[node_numbers[i]]])
         check_contents <- unlist(strsplit(as.character(check_node), "\n"))
@@ -252,7 +284,7 @@ srppp_xml_get_parallel_imports <- function(srppp_xml = srppp_xml_get())
       dimnames = list(NULL, pi_attribute_names)) |>
     as_tibble() |>
     rename(pNbr = packageInsert) |>
-    mutate(across(c(producingCountryPrimaryKey, pNbr), as.integer)) |>
+    mutate(pNbr = as.integer(pNbr)) |>
     arrange(wNbr)
 
   ph_nodes <- xml_find_all(srppp_xml,
@@ -265,7 +297,6 @@ srppp_xml_get_parallel_imports <- function(srppp_xml = srppp_xml_get())
   }))
   colnames(ph_key_matrix) <- c("id", "permission_holder")
   ph_keys <- as_tibble(ph_key_matrix)
-  ph_keys$permission_holder <- as.integer(ph_keys$permission_holder)
 
   # Discard the second permission holder
   # For example, in the XML file from 2019-03-05, the Parallelimport F-6146
@@ -292,17 +323,21 @@ srppp_xml_get_substances <- function(srppp_xml = srppp_xml_get()) {
   substance_nodeset <- xml_find_all(srppp_xml, "MetaData[@name='Substance']/Detail")
 
   sub_desc <- t(sapply(substance_nodeset, function(sub_node) {
-    c(xml_attr(sub_node, "primaryKey"),
+    desc_nodes <- xml_children(sub_node)
+
+    substance_info <- trimws(c(
+      xml_attr(sub_node, "primaryKey"),
       xml_attr(sub_node, "iupacName"),
-      xml_attr(xml_children(sub_node), "value")
-    ) |>
-    trimws()
+      xml_attr(desc_nodes, "value")))
+
+    xml_attr(desc_nodes, "language")
+
+    names(substance_info) <- c("pk", "iupac", paste0("substance_", xml_attr(desc_nodes, "language")))
+    return(substance_info[c("pk", "iupac", paste0("substance_", c("de", "fr", "it", "en")))])
   }))
 
-  colnames(sub_desc) <- c("pk", "iupac", "substance_de", "substance_fr", "substance_it", "substance_en", "substance_lt")
   ret <- as_tibble(sub_desc) |>
-    dplyr::mutate(pk = as.integer(pk)) |>
-    dplyr::arrange(pk)
+    dplyr::arrange(substance_de)
 
   return(ret)
 }
@@ -323,7 +358,7 @@ srppp_xml_get_ingredients <- function(srppp_xml = srppp_xml_get())
 
   get_ingredient_map <- function(ingredient_node) {
     wNbr <- xml_attr(xml_parent(xml_parent(ingredient_node)), "wNbr")
-    pk <- xml_attr(xml_child(ingredient_node, search = 2), "primaryKey")
+    pk <- toupper(xml_attr(xml_child(ingredient_node, search = 2), "primaryKey")) # In some cases we have lower case UUIDs here
     type <- xml_text(xml_child(ingredient_node, search = 1))
     attributes <- xml_attrs(ingredient_node)
     ret <- c(wNbr, pk, type,
@@ -340,9 +375,7 @@ srppp_xml_get_ingredients <- function(srppp_xml = srppp_xml_get())
     mutate(percent = if_else(
       type == "ADDITIVE_TO_DECLARE", "", percent)) |>
     mutate(g_per_L = if_else(
-      type == "ADDITIVE_TO_DECLARE", "", g_per_L)) |>
-    mutate(add_txt_pk = as.integer(add_txt_pk)) |>
-    mutate(pk = as.integer(pk))
+      type == "ADDITIVE_TO_DECLARE", "", g_per_L))
 
   ingredient_descriptions <- srppp_xml |>
     xml_find_all(paste0("MetaData[@name='IngredientAdditionalText']/Detail")) |>
@@ -350,7 +383,6 @@ srppp_xml_get_ingredients <- function(srppp_xml = srppp_xml_get())
     as_tibble() |>
     rename(ingredient_de = de, ingredient_fr = fr) |>
     rename(ingredient_it = it, ingredient_en = en) |>
-    mutate(desc_pk = as.integer(desc_pk)) |>
     rename(ingr_desc_pk = desc_pk) |>
     arrange(ingr_desc_pk)
 
@@ -442,7 +474,6 @@ srppp_xml_get_uses <- function(srppp_xml = srppp_xml_get()) {
     rename(units_de = de, units_fr = fr) |>
     rename(units_it = it, units_en = en) |>
     rename(units_pk = desc_pk) |>
-    mutate(units_pk = as.integer(units_pk)) |>
     arrange(units_pk)
 
   time_units_nodeset <- xml_find_all(srppp_xml, "Products/Product/ProductInformation/Indication/TimeMeasure")
@@ -456,7 +487,7 @@ srppp_xml_get_uses <- function(srppp_xml = srppp_xml_get()) {
   }
   time_units <- t(sapply(time_units_nodeset, get_time_units)) |>
     as_tibble() |>
-    mutate_at(c("use_nr", "time_units_pk"), as.integer)
+    mutate(use_nr = as.integer(use_nr))
 
   time_unit_descriptions <- srppp_xml |>
     xml_find_all(paste0("MetaData[@name='TimeMeasure']/Detail")) |>
@@ -465,13 +496,12 @@ srppp_xml_get_uses <- function(srppp_xml = srppp_xml_get()) {
     rename(time_units_de = de, time_units_fr = fr) |>
     rename(time_units_it = it, time_units_en = en) |>
     rename(time_units_pk = desc_pk) |>
-    mutate(time_units_pk = as.integer(time_units_pk)) |>
     arrange(time_units_pk)
 
   uses <- t(sapply(use_nodeset, get_use)) |>
     as_tibble() |>
     mutate_at(c("min_dosage", "max_dosage", "min_rate", "max_rate"), as.numeric) |>
-    mutate_at(c("waiting_period", "use_nr", "units_pk"), as.integer) |>
+    mutate_at(c("waiting_period", "use_nr"), as.integer) |>
     select(wNbr, use_nr, min_dosage, max_dosage, waiting_period, min_rate, max_rate, units_pk) |>
     left_join(time_units, by = join_by(wNbr, use_nr))
 
@@ -479,8 +509,8 @@ srppp_xml_get_uses <- function(srppp_xml = srppp_xml_get()) {
     left_join(rate_unit_descriptions, by = "units_pk") |>
     left_join(time_unit_descriptions, by = "time_units_pk") |>
     select(wNbr, use_nr, ends_with("dosage"),
-      ends_with("rate"), starts_with("units"),
-      waiting_period, starts_with("time_units"),
+      ends_with("rate"), starts_with("units"), -units_pk,
+      waiting_period, starts_with("time_units"), -time_units_pk,
       starts_with("application_area")) |>
     arrange(wNbr, use_nr)
 
@@ -633,7 +663,7 @@ srppp_dm <- function(from = srppp_xml_url, remove_duplicates = TRUE, verbose = T
 
       ret <- tibble(
           wNbr = sapply(product_information_nodes, get_grandparent_wNbr),
-          desc_pk = as.integer(xml_attr(product_information_nodes, "primaryKey"))
+          desc_pk = xml_attr(product_information_nodes, "primaryKey")
         ) |>
         filter(!grepl("-", wNbr)) |>
         left_join(descriptions, by = "desc_pk") |>
@@ -690,7 +720,6 @@ srppp_dm <- function(from = srppp_xml_url, remove_duplicates = TRUE, verbose = T
     stop("Duplicate entries for the same ingredient in the same product")
   }
 
-
   # Define use IDs (attribute 'use_nr' in the XML tree)
   srppp_xml <- srppp_xml_define_use_numbers(srppp_xml)
 
@@ -705,13 +734,13 @@ srppp_dm <- function(from = srppp_xml_url, remove_duplicates = TRUE, verbose = T
       wNbr = sapply(indication_information_nodes, get_great_grandparent_wNbr),
       use_nr = sapply(indication_information_nodes, get_use_nr),
       desc_pk = xml_attr(indication_information_nodes, "primaryKey")) |>
-        mutate_at(c("use_nr", "desc_pk"), as.integer) |>
+        mutate(use_nr = as.integer(use_nr)) |>
         left_join(products[c("wNbr", "pNbr")], by = "wNbr") |>
         select(pNbr, !c(wNbr, pNbr))
 
     if (additional_text) {
-      ret$add_txt_pk <- as.integer(xml_attr(indication_information_nodes,
-        "additionalTextPrimaryKey"))
+      ret$add_txt_pk <- xml_attr(indication_information_nodes,
+        "additionalTextPrimaryKey")
     }
 
     if (type) {
@@ -896,6 +925,7 @@ srppp_dm <- function(from = srppp_xml_url, remove_duplicates = TRUE, verbose = T
 
     attr(srppp_dm, "from") <- attr(srppp_xml, "from")
     attr(srppp_dm, "culture_tree") <- culture_tree
+    attr(srppp_dm, "srppp_xml_version") <- attr(srppp_xml, "srppp_xml_version")
     class(srppp_dm) <- c("srppp_dm", "dm")
     return(srppp_dm)
 }
@@ -969,7 +999,6 @@ description_table <- function(srppp_xml, tag_name, code = FALSE, latin = FALSE,
     ret <- nodes |>
       sapply(get_descriptions, code = code, latin = latin, parent_keys = parent_keys) |> t() |>
       as_tibble() |>
-      mutate(desc_pk = as.integer(desc_pk)) |>
       arrange(desc_pk)
   } else {
     ret <- NA
@@ -988,13 +1017,14 @@ description_table <- function(srppp_xml, tag_name, code = FALSE, latin = FALSE,
 get_descriptions <- function(node, code = FALSE, latin = FALSE, parent_keys = FALSE) {
   desc_pk <- xml_attr(node, "primaryKey")
   desc <- trimws(sapply(xml_children(node), xml_attr, "value"))
+  names(desc) <- sapply(xml_children(node), xml_attr, "language")
   if (code) {
     if (xml_length(xml_child(node)) == 1) {
       code <- xml_attr(xml_child(xml_child(node)), "value")
     } else {
       code <- NA
     }
-    ret <- c(desc_pk, code, desc)
+    ret <- c(desc_pk, code, desc[c("de", "fr", "it", "en")])
     names(ret) <- c("desc_pk", "code", "de", "fr", "it", "en")
     return(ret)
   } else {
@@ -1008,15 +1038,16 @@ get_descriptions <- function(node, code = FALSE, latin = FALSE, parent_keys = FA
       if (length(pks) > 0) parent_key_1 <- pks[1]
       if (length(pks) > 1) parent_key_2 <- pks[2]
       if (length(pks) > 2) stop("Assumption of a maximum of two parent primary keys for cultures appears to be wrong")
-      ret <- c(desc_pk, desc, parent_key_1, parent_key_2)
+      ret <- c(desc_pk, desc[c("de", "fr", "it", "en")], parent_key_1, parent_key_2)
       names(ret) <- c("desc_pk", "de", "fr", "it", "en", "prt_1_pk", "prt_2_pk")
       return(ret)
     } else {
-      ret <- c(desc_pk, desc)
 
       if (latin) {
+        ret <- c(desc_pk, desc[c("de", "fr", "it", "en", "lt")])
         names(ret) <- c("desc_pk", "de", "fr", "it", "en", "lt")
       } else {
+        ret <- c(desc_pk, desc[c("de", "fr", "it", "en")])
         names(ret) <- c("desc_pk", "de", "fr", "it", "en")
       }
       return(ret)
